@@ -5,6 +5,7 @@ module Llama where
 import Conduit
 import Data.Aeson
 import Data.Text (Text)
+import Data.Word
 import GHC.Generics
 import Network.HTTP.Conduit
 import Network.HTTP.Simple hiding (httpLbs)
@@ -16,20 +17,14 @@ import Llama.Streaming
 data Role = System | User | CustomRole Text deriving Show
 instance ToJSON Role where
   toJSON System = "system"
-  toJSON User = "system"
+  toJSON User = "user"
   toJSON (CustomRole t) = String t
 
 data LlamaMessage = LlamaMessage
   { role :: Role
   , content :: Text
-  } deriving (Show)
-
-instance ToJSON LlamaMessage where
-  toJSON m =
-    object
-      [ "role" .= role m
-      , "content" .= Llama.content m
-      ]
+  } deriving (Show, Generic)
+instance ToJSON LlamaMessage
 
 newtype LlamaApplyTemplateRequest = LlamaApplyTemplateRequest
   { messages :: [LlamaMessage]
@@ -39,10 +34,17 @@ instance ToJSON LlamaApplyTemplateRequest
 newtype LlamaApplyTemplateResponse = LlamaApplyTemplateResponse
   { prompt :: Text
   } deriving (Show, Generic)
+instance FromJSON LlamaApplyTemplateResponse
 
-instance FromJSON LlamaApplyTemplateResponse where
-  parseJSON = withObject "LlamaApplyTemplateResponse" $ \v -> LlamaApplyTemplateResponse
-    <$> v .: "prompt"
+newtype LlamaDetokenizeRequest = LlamaDetokenizeRequest
+  { tokens :: [Token]
+  } deriving (Show, Generic)
+instance ToJSON LlamaDetokenizeRequest
+
+newtype LlamaDetokenizeResponse = LlamaDetokenizeResponse
+  { content :: Text
+  } deriving (Show, Generic)
+instance FromJSON LlamaDetokenizeResponse
 
 data Health = HealthOk | HealthNok deriving (Show)
 
@@ -50,25 +52,15 @@ data Health = HealthOk | HealthNok deriving (Show)
 data LlamaRequest = LlamaRequest
   { prompt :: Text
   , stream :: Bool
-  } deriving (Show)
-
-instance FromJSON LlamaRequest where
-  parseJSON = withObject "LlamaRequest" $ \v -> LlamaRequest
-    <$> v .: "prompt"
-    <*> v .: "stream"
-
-instance ToJSON LlamaRequest where
-  toJSON (LlamaRequest p s) =
-    object ["prompt" .= p, "stream" .= s]
+  } deriving (Show, Generic)
+instance ToJSON LlamaRequest
 
 newtype LlamaResponse = LlamaResponse
-  { generatedText :: Text
-  } deriving (Show)
-
-instance FromJSON LlamaResponse where
-  parseJSON = withObject "LlamaResponse" $ \v -> LlamaResponse
-    <$> v .: "content"
+  { content :: Text
+  } deriving (Show, Generic)
+instance FromJSON LlamaResponse
 
+type Token = Word32
 type URL = String
 
 applyTemplate :: URL -> Manager -> LlamaApplyTemplateRequest -> IO (Maybe Text)
@@ -112,6 +104,21 @@ sendToLlamaStreaming url manager input = do
                     }
   pure $ httpSource req getResponseBody .| eventConduit
 
+detokenize :: URL -> [Token] -> IO (Maybe Text)
+detokenize url input = do
+  let request = parseRequest_ $ url ++ "/detokenize"
+      body = encode $ LlamaDetokenizeRequest input
+      req = request { method = "POST"
+                    , requestBody = RequestBodyLBS body
+                    , requestHeaders = [("Content-Type", "application/json")]
+                    }
+  response <- httpLBS req
+  case decode (responseBody response) of
+    Just (LlamaDetokenizeResponse text) -> return (Just text)
+    Nothing -> do
+      liftIO $ hPutStrLn stderr "Failed to decode Llama response"
+      return Nothing
+
 llama :: URL -> Text -> IO (Maybe Text)
 llama url input = do
   manager <- liftIO $ newManager tlsManagerSettings { managerResponseTimeout = responseTimeoutNone }
