@@ -9,6 +9,7 @@ module Main where
 
 import Conduit
 import Data.Conduit.Lazy
+import qualified Data.Text as T
 import qualified Data.Text.IO as T
 import Options.Generic
 import System.Exit
@@ -40,7 +41,7 @@ main = do
       response <- llamaTemplated (url opts) (LlamaApplyTemplateRequest request)
       case response of
         Nothing -> T.hPutStrLn stderr "Got no response from the server." >> exitFailure
-        Just r -> T.putStrLn r
+        Just r -> T.putStrLn $ if stripThinking opts then snd $ T.breakOnEnd "</think>" r else r
     True -> do
       hSetBuffering stdout NoBuffering
       conduit <- llamaTemplatedStreaming (url opts) (LlamaApplyTemplateRequest request)
